"""
This file is required only for loose coupling and avoiding import loops.
"""
# Author: Gael Varoquaux <gael dot varoquaux at normalesup dot org>
# Copyright: Gael Varoquaux
# License: BSD

__version__ = "0.3.4b"

